* This presumes the meta data has a definition of the table as it was before
* any changes were made.

#define ccCRLF chr(13) + chr(10)
define class PublishTableChanges as Custom

* Public properties.

	cFormat       = 'RSS'
		&& the format for publication
	cErrorMessage = ''
		&& the message of any error
	cLink         = ''
		&& the link for the feed
	cAuthor       = ''
		&& the author for the feed
	cApplication  = ''
		&& the name of the application
	cRSSFileName  = ''
		&& the name of the XML file to create
	cMetaDataPath = ''
		&& the location of the meta data tables
	lChanges      = .F.
		&& .T. if there are any changes that were published

* Protected properties for internal use.

	protected cTable, cAlias, cDatabase, cContent, lUsed, oMeta
	cTable    = ''
	cAlias    = ''
	cDatabase = ''
	cContent  = ''
	lUsed     = .F.
	oMeta     = .NULL.

* Publish changes to the specified table.

	function PublishTableChanges(tcTable)
		local llReturn, ;
			lnSelect
		with This

* Ensure properties are set up correctly.

			do case
				case vartype(.cRSSFileName) <> 'C' or empty(.cRSSFileName)
					.cErrorMessage = 'cRSSFileName was not specified'
					llReturn       = .F.
				case vartype(.cFormat) <> 'C' or ;
					not inlist(.cFormat, 'RSS', 'Atom')
					.cErrorMessage = 'Invalid cFormat property'
					llReturn       = .F.
				case vartype(.cMetaDataPath) <> 'C' or ;
					(empty(.cMetaDataPath) and not file('DBCXReg.dbf')) or ;
					not file(forcepath('DBCXReg.dbf', .cMetaDataPath))
					.cErrorMessage = 'cMetaDataPath was not specified'
					llReturn       = .F.
				otherwise
					.oMeta = newobject('DBCXMgr', 'DBCXMgr.vcx', '', .F., ;
						.cMetaDataPath)
					llReturn = vartype(.oMeta) = 'O'
			endcase			

* If everything is OK, open the table.

			if llReturn
				lnSelect = select()
				.cTable  = tcTable
				.cAlias  = juststem(tcTable)
				llReturn = .OpenTable()

* If we opened the table, check the table, fields, and indexes, then close
* the table and publish any changes.

				if llReturn
					.cDatabase = juststem(cursorgetprop('Database'))
					.cContent  = ''
					.CheckTable()
					.CheckFields()
					.CloseTable()
					if not empty(.cContent)
						.EndContent()
						.Publish()
						.lChanges = .T.
					endif not empty(.cContent)
					.oMeta.Release()
				endif llReturn
				select (lnSelect)
			endif llReturn
		endwith
		return llReturn
	endfunc

	protected function CheckTable
*** Left for an exercise for the reader: check table-level changes, such as
*** codepage and blocksize.
	endfunc

	protected function CheckFields
		local laFields[1], ;
			lnFields, ;
			lnI, ;
			lcField, ;
			lcSize, ;
			lcDec, ;
			loProps, ;
			lcChange, ;
			llChanged, ;
			lcNewType, ;
			lcNewStru, ;
			lcOldType, ;
			lcOldStru, ;
			laMetaFields[1]

* Go through all fields in the table looking for changes. We'll use DBCXMgr's
* DBCXGetPropertyObject method to get an object with the meta data properties
* for each field as they were prior to the changes, then compare that to the
* actual structure.

		with This
			lnFields = afields(laFields)
			for lnI = 1 to lnFields
				lcField = laFields[lnI, 1]
				lcSize  = transform(laFields[lnI, 3])
				lcDec   = transform(laFields[lnI, 4])
				loProps = .oMeta.DBCXGetPropertyObject(.cDatabase + '!' + ;
					.cAlias + '.' + lcField, 'Field')

* The field existed before, so look for any changes. We'll create strings with
* the old structure and the new one and compare them.

				if vartype(loProps) = 'O'
					lcChange  = 'Changes made to field ' + lcField + ':' + ;
						ccCRLF + '&lt;ul&gt;' + ccCRLF
					llChanged = .F.
					lcNewType = laFields[lnI, 2]
					lcNewStru = lcNewType
					do case
						case lcNewType $ 'CVQ'
							lcNewStru = lcNewStru + '(' + lcSize + ')'
						case lcNewType $ 'NF'
							lcNewStru = lcNewStru + '(' + lcSize + ',' + ;
								lcDec + ')'
						case lcNewType = 'B'
							lcNewStru = lcNewStru + '(' + lcDec + ')'
					endcase
					lcOldType = loProps.cType
					lcOldStru = lcOldType
					do case
						case lcOldType $ 'CVQ'
							lcOldStru = lcOldStru + '(' + ;
								transform(loProps.nSize) + ')'
						case lcOldType $ 'NF'
							lcOldStru = lcOldStru + '(' + ;
								transform(loProps.nSize) + ',' + ;
								transform(loProps.nDecimals) + ')'
						case lcOldType = 'B'
							lcOldStru = lcOldStru + '(' + ;
								transform(loProps.nDecimals) + ')'
					endcase

* If the structure changed, log it.

					if lcOldStru <> lcNewStru
						lcChange  = lcChange + '&lt;li&gt;Structure ' + ;
							'changed from ' + lcOldStru + ' to ' + ;
							lcNewStru + '&lt;/li&gt;' + ccCRLF
						llChanged = .T.
					endif lcOldStru <> lcNewStru

* Check for null and code page translation changes.

					if loProps.lNull <> laFields[lnI, 5]
						lcChange  = lcChange + '&lt;li&gt;Nulls turned ' + ;
							iif(loProps.lNull, 'off', 'on') + '&lt;/li&gt;' + ;
							ccCRLF
						llChanged = .T.
					endif loProps.lNull <> laFields[lnI, 5]
					if loProps.lBinary <> laFields[lnI, 6]
						lcChange  = lcChange + '&lt;li&gt;Code page ' + ;
							'translation turned ' + ;
							iif(loProps.lBinary, 'on', 'off') + ;
							'&lt;/li&gt;' + ccCRLF
						llChanged = .T.
					endif loProps.lBinary <> laFields[lnI, 6]
					do case

* This field is an auto-incrementing field now but it wasn't before.

						case loProps.nStepValue <> laFields[lnI, 18] and ;
							loProps.nStepValue = 0
							lcChange  = lcChange + '&lt;li&gt;' + ;
								'Auto-incrementing turned on&lt;/li&gt;' + ;
								ccCRLF
							llChanged = .T.

* This field isn't an auto-incrementing field now but it was before.

						case loProps.nStepValue <> laFields[lnI, 18] and ;
							laFields[lnI, 18] = 0
							lcChange  = lcChange + '&lt;li&gt;' + ;
								'Auto-incrementing turned off&lt;/li&gt;' + ;
								ccCRLF
							llChanged = .T.

* The step value for the auto-incrementing field changed (we don't check for
* the next value changing since that's irrelevant from a meta data viewpoint).

						case loProps.nStepValue <> laFields[lnI, 18]
							lcChange  = lcChange + '&lt;li&gt;' + ;
								'Auto-incrementing step value changed from ' + ;
								transform(loProps.nStepValue) + ' to ' + ;
								transform(laFields[lnI, 18]) + '&lt;/li&gt;' + ;
								ccCRLF
							llChanged = .T.
					endcase

* If anything changed, update the log content.

					if llChanged
						.UpdateContent(lcChange + '&lt;/ul&gt;' + ccCRLF)
					endif llChanged

* This is a new field, so log it.

				else
					.UpdateContent('Field ' + lcField + ' added')
				endif vartype(loProps) = 'O'
			next lnI

* Now look for fields in the meta data that are no longer in the table (ie.
* deleted or renamed fields).

			lnFields = loMeta.DBCXGetAllObjects('Field ' + .cAlias, ;
				@laMetaFields)
			for lnI = 1 to lnFields
				lcField = laMetaFields[lnI]
				if ascan(laFields, lcField, -1, -1, 1, 7) = 0
					.UpdateContent('Field ' + lcField + ' deleted')
				endif ascan(laFields, lcField, -1, -1, 1, 7) = 0
			next lnI
		endwith
	endfunc

	protected function CheckIndexes
*** Left for an exercise for the reader: do similar steps in CheckFields
*** using ATAGINFO()
	endfunc

	protected function UpdateContent(tcChange)
		with This
			if empty(.cContent)
				.StartContent()
			endif empty(.cContent)
			.cContent = .cContent + '&lt;tr&gt;&lt;td&gt;' + tcChange + ;
				'&lt;/td&gt;&lt;/tr&gt;' + ;
				ccCRLF
		endwith
	endfunc

	protected function StartContent
		This.cContent = '&lt;table&gt;' + ccCRLF
	endfunc

	protected function EndContent
		This.cContent = This.cContent + '&lt;table&gt;' + ccCRLF
	endfunc

	protected function Publish
		local loRSS as SFRSSGeneratorRSS2 of SFRSS.VCX, ;
			loItem as SFRSSItem of SFRSS.VCX
		with This
			loRSS = .CreateRSSGenerator()
			loRSS.Title       = 'Data Dictionary Update'
			loRSS.Description = 'Data dictionary changes were made to ' + ;
				.cApplication
			loRSS.Link        = .cLink

			loItem = loRSS.AddItem()
			loItem.Title       = 'Changes made to ' + .cTable
			loItem.Author      = .cAuthor
			loItem.Description = .cContent

			strtofile(loRSS.GetRSS(), .cRSSFileName)
		endwith
	endfunc

	protected function CreateRSSGenerator
		local loRSS
		with This
			do case
				case .cFormat = 'RSS'
					loRSS = newobject('SFRSSGeneratorRSS2', 'SFRSS.VCX')
*** Handle other subclasses here
				otherwise
					loRSS          = .NULL.
					.cErrorMessage = 'Invalid cFormat property'
			endcase
		endwith
		return loRSS
	endfunc

	protected function OpenTable
		local loException as Exception, ;
			llReturn
		with This
			.lUsed = used(.cAlias)
			if .lUsed
				select (.cAlias)
			else
				try
					use (.cTable) again shared
				catch to loException
					.cErrorMessage = loException.Message
				endtry
			endif .lUsed
			llReturn = used(.cAlias)
		endwith
		return llReturn
	endfunc

	protected function CloseTable
		with This
			if not .lUsed
				use in select(.cAlias)
			endif not .lUsed
		endwith
	endfunc
enddefine
